# Copyright © 2015 STRG.AT GmbH, Vienna, Austria
#
# This file is part of the The SCORE Framework.
#
# The SCORE Framework and all its parts are free software: you can redistribute
# them and/or modify them under the terms of the GNU Lesser General Public
# License version 3 as published by the Free Software Foundation which is in the
# file named COPYING.LESSER.txt.
#
# The SCORE Framework and all its parts are distributed without any WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. For more details see the GNU Lesser General Public
# License.
#
# If you have not received a copy of the GNU Lesser General Public License see
# http://www.gnu.org/licenses/.
#
# The License-Agreement realised between you as Licensee and STRG.AT GmbH as
# Licenser including the issue of its valid conclusion and its pre- and
# post-contractual effects is governed by the laws of Austria. Any disputes
# concerning this License-Agreement including the issue of its valid conclusion
# and its pre- and post-contractual effects are exclusively decided by the
# competent court, in whose district STRG.AT GmbH has its registered seat, at
# the discretion of STRG.AT GmbH also the competent court, in whose district the
# Licensee has his registered seat, an establishment or assets.

import os
from setuptools import setup
from setuptools.command.install import install
from setuptools.command.develop import develop
import textwrap
import sys


here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.rst')) as f:
    README = f.read()


class ShellUpdateMixin:

    def run(self):
        result = super().run()
        is_virtualenv = (
            hasattr(sys, 'real_prefix') or (
                hasattr(sys, 'base_prefix') and sys.base_prefix != sys.prefix))
        if not is_virtualenv:
            # we are outside a virtual environment
            # FIXME: we should be checking, if this is a --user installation,
            # but we have not found a reliable, cross-platform way of doing that
            # TODO: maybe we should create a bashrc, if there is none?
            self._update_bashrc() or self._update_bash_profile()
            self._update_zshrc()
        return result

    def _update_bashrc(self):
        return self._update_rc_file(os.path.expanduser('~/.bashrc'),
                                    self._create_bash_prompt())

    def _update_bash_profile(self):
        return self._update_rc_file(os.path.expanduser('~/.bash_profile'),
                                    self._create_bash_prompt())

    def _create_bash_prompt(self):
        return textwrap.dedent(r'''
            if [ -n "$VIRTUAL_ENV_NAME" ]; then
              export PS1="\[[0;33m\](${VIRTUAL_ENV_NAME})\[[0m\] $PS1"
            elif [ -n "$VIRTUAL_ENV" ]; then
              export PS1="\[[0;33m\](${VIRTUAL_ENV##*/})\[[0m\] $PS1"
            fi
        ''').strip()

    def _update_zshrc(self):
        prompt = textwrap.dedent(r'''
            if [ -n "$VIRTUAL_ENV_NAME" ]; then
              export PROMPT="%{[0;33m%}(${VIRTUAL_ENV_NAME})%{[0m%} $PROMPT"
            elif [ -n "$VIRTUAL_ENV" ]; then
              export PROMPT="%{[0;33m%}(${VIRTUAL_ENV##*/})%{[0m%} $PROMPT"
            fi
        ''').strip()
        return self._update_rc_file(os.path.expanduser('~/.zshrc'), prompt)

    def _update_rc_file(self, rcfile, prompt):
        try:
            content = open(rcfile).read()
        except FileNotFoundError:
            return False
        # skip the update if there is something similar
        if 'VIRTUAL_ENV_NAME' in content:
            return True
        code = '\n'
        if content[-1] != '\n':
            code += '\n'
        code += textwrap.dedent(r'''
            # The next block was inserted by the `projects' module of
            # The SCORE Framework (http://score-framework.org)

              # This next line updates your shell prompt to include the name of
              # the current project.
        ''').lstrip()
        code += textwrap.indent(prompt, '  ') + '\n'
        open(rcfile, 'a').write(code)
        return True


class InstallCommand(ShellUpdateMixin, install):
    pass


class DevelopCommand(ShellUpdateMixin, develop):
    pass


setup(
    name='score.projects',
    version='0.0.6',
    description='Manage your projects with The SCORE Framework',
    long_description=README,
    author='strg.at',
    author_email='score@strg.at',
    url='http://score-framework.org',
    keywords='score framework project',
    packages=['score', 'score.projects'],
    include_package_data=True,
    namespace_packages=['score'],
    zip_safe=False,
    license='LGPL',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General '
            'Public License v3 or later (LGPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
    ],
    install_requires=[
        'score.cli',
        'score.init',
        'vex',
    ],
    entry_points={
        'score.cli': [
            'projects = score.projects.cli:main',
        ]
    },
    cmdclass={
        'install': InstallCommand,
        'develop': DevelopCommand,
    }
)
